@@ -7,6 +7,8 @@
 #include "network-route.h"
 #include "mnl_util.h"
 #include "network-util.h"
+#include "netlink-missing.h"
+#include "string-util.h"
 
 int route_new(Route **ret) {
         Route *route;
@@ -44,7 +46,7 @@ static int routes_new(Routes **ret) {
         if (r < 0)
                 return r;
 
-        *ret = steal_pointer(rt);
+        *ret = steal_ptr(rt);
         return 0;
 }
 
@@ -94,6 +96,189 @@ static int route_add(Routes **rts, Route *rt) {
         return -EEXIST;
 }
 
+int route_table_to_string(uint32_t table, char **ret) {
+        _auto_cleanup_ char *str = NULL;
+        const char *s;
+        int r;
+
+        s = route_table_to_name(table);
+        if (!s)
+                r = asprintf(&str, "%" PRIu32, table);
+        else
+                r = asprintf(&str, "%s(%" PRIu32 ")", s, table);
+        if (r < 0)
+                return -ENOMEM;
+
+        *ret = steal_ptr(str);
+        return 0;
+}
+
+static const char * const route_table[_ROUTE_TABLE_MAX] = {
+       [ROUTE_TABLE_LOCAL]    = "local",
+       [ROUTE_TABLE_MAIN]     = "main",
+       [ROUTE_TABLE_DEFAULT]  = "default",
+};
+
+const char *route_table_to_name(int id) {
+        if (id < 0)
+                return NULL;
+
+        if ((size_t) id >= ELEMENTSOF(route_table))
+                return NULL;
+
+        return route_table[id];
+}
+
+int route_table_to_mode(const char *name) {
+        assert(name);
+
+        for (size_t i = ROUTE_TABLE_DEFAULT; i < (size_t) ELEMENTSOF(route_table); i++)
+                if (str_eq_fold(name, route_table[i]))
+                        return i;
+
+        return _ROUTE_TABLE_INVALID;
+}
+
+static const char *const route_scope_type[_ROUTE_SCOPE_MAX] =  {
+        [ROUTE_SCOPE_UNIVERSE] = "global",
+        [ROUTE_SCOPE_SITE]     = "site",
+        [ROUTE_SCOPE_LINK]     = "link",
+        [ROUTE_SCOPE_HOST]     = "host",
+        [ROUTE_SCOPE_NOWHERE]  = "nowhere",
+};
+
+const char *route_scope_type_to_name(int id) {
+        if (id < 0)
+                return NULL;
+
+        if ((size_t) id >= ELEMENTSOF(route_scope_type))
+                return NULL;
+
+        return route_scope_type[id];
+}
+
+int route_scope_type_to_mode(const char *name) {
+        assert(name);
+
+        for (size_t i = ROUTE_SCOPE_UNIVERSE; i < (size_t) ELEMENTSOF(route_scope_type); i++)
+                if (route_scope_type[i] && str_eq_fold(name, route_scope_type[i]))
+                        return i;
+
+        return _ROUTE_SCOPE_INVALID;
+}
+
+static const char * const route_type[_ROUTE_TYPE_MAX] = {
+        [ROUTE_TYPE_UNICAST]     = "unicast",
+        [ROUTE_TYPE_LOCAL]       = "local",
+        [ROUTE_TYPE_BROADCAST]   = "broadcast",
+        [ROUTE_TYPE_ANYCAST]     = "anycast",
+        [ROUTE_TYPE_MULTICAST]   = "multicast",
+        [ROUTE_TYPE_BLACKHOLE]   = "blackhole",
+        [ROUTE_TYPE_UNREACHABLE] = "unreachable",
+        [ROUTE_TYPE_PROHIBIT]    = "prohibit",
+        [ROUTE_TYPE_THROW]       = "throw",
+        [ROUTE_TYPE_NAT]         = "nat",
+        [ROUTE_TYPE_XRESOLVE]    = "xresolve",
+};
+
+const char *route_type_to_name(int id) {
+        if (id < 0)
+                return NULL;
+
+        if ((size_t) id >= ELEMENTSOF(route_type))
+                return NULL;
+
+        return route_type[id];
+}
+
+int route_type_to_mode(const char *name) {
+        assert(name);
+
+        for (size_t i = ROUTE_TYPE_UNICAST; i < (size_t) ELEMENTSOF(route_type); i++)
+                if (route_type[i] && str_eq_fold(name, route_type[i]))
+                        return i;
+
+        return _ROUTE_TYPE_INVALID;
+}
+
+static const char * const ipv6_route_preference_type[_IPV6_ROUTE_PREFERENCE_MAX] = {
+        [IPV6_ROUTE_PREFERENCE_LOW]     = "low",
+        [IPV6_ROUTE_PREFERENCE_MEDIUM]  = "medium",
+        [IPV6_ROUTE_PREFERENCE_HIGH]    = "high",
+};
+
+const char *ipv6_route_preference_to_name(int id) {
+        if (id < 0)
+                return NULL;
+
+        if ((size_t) id >= ELEMENTSOF(ipv6_route_preference_type))
+                return NULL;
+
+        return ipv6_route_preference_type[id];
+}
+
+int ipv6_route_preference_type_to_mode(const char *name) {
+        assert(name);
+
+        for (size_t i = IPV6_ROUTE_PREFERENCE_LOW; i < (size_t) ELEMENTSOF(ipv6_route_preference_type); i++)
+                if (str_eq_fold(name, ipv6_route_preference_type[i]))
+                        return i;
+
+        return _IPV6_ROUTE_PREFERENCE_INVALID;
+}
+
+static const char * const route_protocol_type[_ROUTE_PROTOCOL_MAX] = {
+       [ROUTE_PROTOCOL_KERNEL]  = "kernel",
+       [ROUTE_PROTOCOL_BOOT]    = "boot",
+       [ROUTE_PROTOCOL_STATIC]  = "static",
+       [ROUTE_PRTOCOL_DHCP]     = "dhcp",
+};
+
+const char *route_protocol_to_name(int id) {
+        if (id < 0)
+                return NULL;
+
+        if ((size_t) id >= ELEMENTSOF(route_protocol_type))
+                return NULL;
+
+        return route_protocol_type[id];
+}
+
+int route_protocol_to_mode(const char *name) {
+        assert(name);
+
+        for (size_t i = IPV6_ROUTE_PREFERENCE_LOW; i < (size_t) ELEMENTSOF(route_protocol_type); i++)
+                if (str_eq_fold(name, route_protocol_type[i]))
+                        return i;
+
+        return _ROUTE_PROTOCOL_INVALID;
+}
+
+static const char * const ipoib_mode_table[_IP_OIB_MODE_MODE_MAX] = {
+        [IP_OIB_MODE_DATAGRAM]       = "datagram",
+        [IP_OIB_MODE_MODE_CONNECTED] = "connected",
+};
+
+const char *ipoib_mode_to_name(int id) {
+        if (id < 0)
+                return NULL;
+
+        if ((size_t) id >= ELEMENTSOF(ipoib_mode_table))
+                return NULL;
+
+        return ipoib_mode_table[id];
+}
+
+int ipoib_name_to_mode(const char *name) {
+        assert(name);
+
+        for (size_t i = IP_OIB_MODE_DATAGRAM; i < (size_t) ELEMENTSOF(ipoib_mode_table); i++)
+                if (str_eq_fold(name, ipoib_mode_table[i]))
+                        return i;
+
+        return _IP_OIB_MODE_MODE_INVALID;
+}
+
 static int validata_attr_mettrics(const struct nlattr *attr, void *data) {
         const struct nlattr **tb = data;
 
@@ -171,7 +356,6 @@ static int fill_link_route_message(Route *rt, int ifindex , struct nlattr *tb[])
         if (tb[RTA_TABLE])
                 rt->table = mnl_attr_get_u32(tb[RTA_TABLE]);
 
-
         if (tb[RTA_DST]) {
                 if (rt->family == AF_INET)
                         memcpy(&rt->dst.in, mnl_attr_get_payload(tb[RTA_DST]), sizeof(struct in_addr));
@@ -193,9 +377,15 @@ static int fill_link_route_message(Route *rt, int ifindex , struct nlattr *tb[])
         if (tb[RTA_OIF])
                 rt->ifindex = mnl_attr_get_u32(tb[RTA_OIF]);
 
+        if (tb[RTA_IIF])
+                rt->iif = mnl_attr_get_u32(tb[RTA_IIF]);
+
         if (ifindex > 0 && ifindex != (int) rt->ifindex)
                 return -EINVAL;
 
+        if (tb[RTA_PREF])
+                rt->pref = mnl_attr_get_u8(tb[RTA_PREF]);
+
         if (tb[RTA_FLOW])
                 rt->flow = mnl_attr_get_u32(tb[RTA_FLOW]);
 
@@ -220,6 +410,9 @@ static int fill_link_route_message(Route *rt, int ifindex , struct nlattr *tb[])
         if (tb[RTA_PRIORITY])
                 rt->priority = mnl_attr_get_u32(tb[RTA_PRIORITY]);
 
+        if (tb[RTA_PRIORITY])
+                rt->ttl_propogate = mnl_attr_get_u8(tb[RTA_PRIORITY]);
+
         if (tb[RTA_METRICS]) {
                 struct nlattr *tbx[RTAX_MAX+1] = {};
                 int i;
@@ -235,7 +428,7 @@ static int fill_link_route_message(Route *rt, int ifindex , struct nlattr *tb[])
 }
 
 static int fill_link_route(const struct nlmsghdr *nlh, void *data) {
-        struct nlattr *tb[RTA_MAX+1] = {};
+        struct nlattr *tb[RTA_MAX * 2] = {};
         _auto_cleanup_ Route *rt = NULL;
         Routes *rts = (Routes *) data;
         struct rtmsg *rm;
@@ -279,7 +472,7 @@ static int fill_link_route(const struct nlmsghdr *nlh, void *data) {
         if (r < 0)
                 return r;
 
-        steal_pointer(rt);
+        steal_ptr(rt);
 
         return MNL_CB_OK;
 }
@@ -315,11 +508,11 @@ static int acquire_link_route(int ifindex, Routes **ret) {
         return 0;
 }
 
-int manager_link_get_routes(Routes **rt) {
+int netlink_acquire_all_link_routes(Routes **rt) {
         return acquire_link_route(0, rt);
 }
 
-int manager_get_one_link_route(int ifindex, Routes **ret) {
+int netlink_get_one_link_route(int ifindex, Routes **ret) {
         return acquire_link_route(ifindex, ret);
 }
 
@@ -382,7 +575,7 @@ static int link_add_route(int s, Route *route) {
         return netlink_call(s, &m->hdr, m->buf, sizeof(m->buf));
 }
 
-int manager_link_add_default_gateway(Route *route) {
+int netlink_add_link_default_gateway(Route *route) {
        _auto_cleanup_close_ int s = -1;
        int r;
 
@@ -395,7 +588,7 @@ int manager_link_add_default_gateway(Route *route) {
         return link_add_route(s, route);
 }
 
-int manager_link_add_route(Route *route) {
+int netlink_add_link_route(Route *route) {
        _auto_cleanup_close_ int s = -1;
        int r;
 
