@@ -1,4 +1,4 @@
-/* Copyright 2022 VMware, Inc.
+/* Copyright 2023 VMware, Inc.
  * SPDX-License-Identifier: Apache-2.0
  */
 #pragma once
@@ -10,7 +10,9 @@ typedef struct Set {
 } Set;
 
 int set_new(Set **ret, GHashFunc hash_func, GEqualFunc compare_func);
-void set_unrefp(Set **s);
+void set_free(Set *s);
+
+DEFINE_CLEANUP(Set *, set_free);
 
 #define set_size(s)  g_hash_table_size(s->hash)
 
